@@ -996,7 +996,7 @@ Player::Run() noexcept
 			}
 		}
 
-		if (dc.IsIdle() && queued && dc.pipe == pipe) {
+		if (dc.IsIdle() && queued && IsDecoderAtCurrentSong()) {
 			/* the decoder has finished the current song;
 			   make it decode the next song */
 
@@ -1058,6 +1058,16 @@ Player::Run() noexcept
 
 			SongBorder();
 		} else if (dc.IsIdle()) {
+			if (queued)
+				/* the decoder has just stopped,
+				   between the two IsIdle() checks,
+				   probably while UnlockCheckOutputs()
+				   left the mutex unlocked; to restart
+				   the decoder instead of stopping
+				   playback completely, let's re-enter
+				   this loop */
+				continue;
+
 			/* check the size of the pipe again, because
 			   the decoder thread may have added something
 			   since we last checked */
